﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using FirebirdSql.Data.FirebirdClient;

namespace SkladEvidence
{
    /// <summary>
    /// Správa uživatelů.
    /// Číslování chyb: 6000 - 6999.
    /// </summary>
    internal partial class Dodavatele : Form, IForm
    {
        private bool bKonec = false, bHistorie = false, bEdit = false,
        bInsert = false, bBlokZmenaRadku = false, bEdAdmin = false,
        bZmena = false;
        private int iId = -1, iRowInd = -1;
        private Util util;

        public Dodavatele(Util pUtil)
        {
            InitializeComponent();
            this.util = pUtil;
            this.Edit = this.Historie = false;
        }

        public bool LzeUkoncit
        {
            get
            {
                if (this.Edit)
                {
                    switch (this.util.PisLog("Dodavatele: Ve formuláři jsou " +
                      "rozpracovaná data. Chcete data uložit?", true,
                      MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question))
                    {
                        case DialogResult.Yes:
                            if (!this.UlozData())
                                return false;
                            break;

                        case DialogResult.No:
                            this.Edit = false;
                            break;

                        default:
                            return false;
                    }
                }

                return true;
            }
        }

        public bool Konec
        {
            set
            {
                bKonec = value;
            }
        }

        private void Dodavatele_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.Cancel = !this.bKonec)
                this.Hide();
        }

        private void Dodavatele_Load(object sender, EventArgs e)
        {
            // Umístím tlačítka na správná místa.
            this.btnOK.Location = this.btnZavritHist.Location =
              this.btnVlozit.Location;
            this.btnStorno.Location = this.btnUpravit.Location;
            this.NactiData();
        }

        private bool Edit
        {
            get
            {
                return this.bEdit;
            }
            set
            {
                this.bEdit = this.btnOK.Visible = this.btnStorno.Visible = value;

                this.btnHistorie.Visible = this.btnObnovit.Visible =
                  this.chbZrusene.Visible = this.tbNazev.ReadOnly = this.tbIco.ReadOnly =
                  this.tbDic.ReadOnly = this.tbPrijmeni.ReadOnly = this.tbJmeno.ReadOnly =
                  this.tbUlice.ReadOnly = this.tbMesto.ReadOnly =
                  this.tbPsc.ReadOnly = this.tbTelefon.ReadOnly = this.tbEmail.ReadOnly = !value;

                this.btnVlozit.Visible = this.btnUpravit.Visible =
                this.btnZrusit.Visible = !this.chbZrusene.Checked && !value;
            }
        }

        private bool Historie
        {
            get
            {
                return this.bHistorie;
            }
            set
            {
                this.bHistorie = this.btnZavritHist.Visible = value;

                this.btnVlozit.Visible = this.btnUpravit.Visible =
                  this.btnZrusit.Visible = !this.chbZrusene.Checked && !value;

                this.btnHistorie.Visible = this.chbZrusene.Visible = !value;
            }
        }

        private bool NactiData()
        {
            int iErrKod = 6000;
            FbCommand fbCmd = null;
            FbDataAdapter fbAda = null;
            string sErr, sPole;
            int i1, iRInd, iCInd;

            try
            {
                // Po dobu načítání dat se nebude evidovat změna aktuálního řádku gridu.
                this.bBlokZmenaRadku = true;

                if (!this.util.PripojDb(out fbCmd, out sErr))
                {
                    iErrKod = 6001;
                    this.util.PisLog("Uzivatele " + iErrKod.ToString() + ": " + sErr,
                      true);
                    return false;
                }

                iRInd = (this.gr?.FirstDisplayedCell?.RowIndex ?? -1);

                iCInd = (this.gr?.FirstDisplayedCell?.ColumnIndex ?? -1);
                fbCmd.Parameters.Clear();
                this.tabDod.Clear();

                sPole = "KDYZMENIL, ID, NAZEV, ICO, DIC, PRIJMENI, JMENO, ULICE, " +
                  "MESTO, PSC, TELEFON, EMAIL, KDOZMENIL ";

                iErrKod = 60002;
                if (!this.Historie)
                {
                    if (!this.chbZrusene.Checked)
                    {
                        iErrKod = 60003;
                        // Všechny záznamy z tabulky TDODAVATELE.
                        fbCmd.CommandText = "select " + sPole +
                          "from tdodavatele order by nazev";
                    }
                    else
                    {
                        iErrKod = 6004;
                        /*
                        // Zrušené záznamy (nejsou v tabulce TDODAVATELE). Vyberu záznamy
                        // s nejnovějším datem změny, tj. datem zrušení.
                        fbCmd.CommandText = "select " + sPole +
                          "from tdodavatelehis uh where not exists (select 1 " +
                          "from tdodavatele u where uh.id = u.id) and " +
                          "uh.kdyzmenil = (select max(kdyzmenil) from tdodavatele uh1 " +
                          "where uh.id = uh1.id)";*/

                        // Zrušené záznamy mají v historické tabulce B_ZRUS = 'A'.
                        fbCmd.CommandText = "select " + sPole +
                          "from tdodavatelehis uh where b_zrus = 'A'";
                    }
                }
                else
                {
                    iErrKod = 6005;
                    // Záznamy z tabulek TUZIVATELE a TUZIVATELEHIS týkající se daného
                    // id.
                    fbCmd.CommandText = "select " + sPole +
                      "from tdodavatele where id = @id " +
                      "union select " + sPole +
                      "from tdodavatelehis where id = @id " +
                      "order by 1 desc";
                    fbCmd.Parameters.Add("@id", FbDbType.Integer).Value = this.iId;
                }

                fbAda = new FbDataAdapter(fbCmd);
                fbAda.Fill(this.tabDod);

                // Pokusím se nastavit na stejné místo v gridu jako před načítáním dat.
                if (iRInd >= 0 && iCInd >= 0 && iRInd < this.gr.RowCount &&
                  iCInd < this.gr.ColumnCount)
                    this.gr.FirstDisplayedCell = this.gr[iCInd, iRInd];

                if (this.iId > 0)
                {
                    for (i1 = 0; i1 < this.gr.RowCount &&
                      (int)this.gr["id", i1].Value != this.iId; ++i1) ;

                    if (i1 < this.gr.RowCount)
                        this.gr.CurrentCell =
                          this.gr[(iCInd >= 0 && iCInd < this.gr.ColumnCount ? iCInd : 0),
                          i1];
                }

                return true;
            }
            catch (Exception exc)
            {
                this.util.PisLog("Dodavatele " + iErrKod.ToString() + ": " +
                  exc.ToString(), true);

                return false;
            }
            finally
            {
                this.bBlokZmenaRadku = false;
                // Musím zajistit aktualizaci iId.
                this.iRowInd = -1;
                this.ZmenaRadku();

                if (fbAda != null)
                    fbAda.Dispose();

                this.util.OdpojDb(fbCmd);
            }
        }

        private void Dodavatele_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.F5)
                this.NactiData();
        }

        private void btnObnovit_Click(object sender, EventArgs e)
        {
            this.NactiData();
        }

        private void chbZrusene_CheckedChanged(object sender, EventArgs e)
        {
            this.btnVlozit.Visible = this.btnUpravit.Visible =
              this.btnZrusit.Visible = !this.chbZrusene.Checked;
            this.NactiData();
        }

        private void btnHistorie_Click(object sender, EventArgs e)
        {
            this.Historie = true;
            this.NactiData();
        }

        private void btnZavritHist_Click(object sender, EventArgs e)
        {
            this.Historie = false;
            this.NactiData();
        }

        private void btnVlozit_Click(object sender, EventArgs e)
        {
            if (!this.util.ClsUziv.bStanice && !this.util.ClsUziv.bAdmin)
            {
                this.util.PisLog("Vkládat dodavatele může administrátor, nebo správa stanice .", true);
                return;
            }

            this.bEdAdmin = false;
            this.Edit = this.bInsert = true;

            this.tbNazev.Text = this.tbIco.Text = this.tbDic.Text = this.tbPrijmeni.Text = this.tbJmeno.Text =
              this.tbUlice.Text = this.tbMesto.Text = this.tbPsc.Text = this.tbTelefon.Text =
              this.tbEmail.Text = "";
        }

        private void btnUpravit_Click(object sender, EventArgs e)
        {
            if (!this.util.ClsUziv.bStanice && !this.util.ClsUziv.bAdmin)
            {
                this.util.PisLog("Editovat dodavatele může administrátor, nebo správa stanice.", true);
                return;
            }

            this.bInsert = false;

            this.Edit = true;
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            this.UlozData();
        }

        private bool UlozData()
        {
            string sErr;
            FbCommand fbCmd = null;
            FbParameter fbParOut = null;
            int iErrKod = 6100;

            try
            {
                iErrKod = 6101;
                if (this.tbNazev.Text == "")
                {
                    this.util.PisLog("Název dodavatele musí být zadán.", true);
                    this.tbNazev.Focus();
                    return false;
                }

                iErrKod = 6102;
                if (this.tbTelefon.Text == "")
                {
                    this.util.PisLog("Telefon dodavatele musí být zadán.", true);
                    this.tbNazev.Focus();
                    return false;
                }


                iErrKod = 6103;
                if (this.tbPsc.Text == "")
                {
                    this.util.PisLog("PSČ dodavatele musí být zadán.", true);
                    this.tbNazev.Focus();
                    return false;
                }

                iErrKod = 6104;
                if (!this.util.PripojDb(out fbCmd, out sErr))
                {
                    iErrKod = 6105;
                    this.util.PisLog("Dodavatele " + iErrKod.ToString() + ": " + sErr,
                      true);
                    return false;
                }

                if (this.bInsert)
                {
                    iErrKod = 6106;
                    fbCmd.CommandText = "insert into tdodavatele (NAZEV, ICO, " +
                      "DIC, JMENO, PRIJMENI, ULICE, MESTO, PSC, TELEFON, " +
                      "EMAIL, KDOZMENIL) " +
                      "values (@NAZEV, @ICO, @DIC, @JMENO, @PRIJMENI, @ULICE, " +
                      "@MESTO, @PSC, @TELEFON, @EMAIL, @KDOZMENIL) " +
                      "returning id";
                    // Parametr pro vrácení id vložené věty.
                    
                    fbParOut = new FbParameter();
                    fbParOut.Direction = ParameterDirection.Output;
                    fbParOut.DbType = DbType.Int32;
                    fbParOut.SourceColumn = "id";
                    fbCmd.Parameters.Add(fbParOut);
                }
                else
                {
                    iErrKod = 6107;
                    fbCmd.CommandText = "update tdodavatele set NAZEV = @nazev, " +
                      "ICO = @ico, DIC = @dic, JMENO = @jmeno, PRIJMENI = @prijmeni, " +
                      "ULICE = @ulice, MESTO = @mesto, " +
                      "PSC = @psc, TELEFON = @telefon, EMAIL = @email," +
                      "KDOZMENIL = @kdozmenil " +
                      "where id = @id";
                    fbCmd.Parameters.Add("@id", FbDbType.Integer).Value = this.iId;
                }

                fbCmd.Parameters.Add("@nazev", FbDbType.VarChar, 30).Value =
                  this.tbNazev.Text.Trim();
                fbCmd.Parameters.Add("@ico", FbDbType.VarChar, 30).Value =
                  this.tbIco.Text.Trim();
                fbCmd.Parameters.Add("@dic", FbDbType.VarChar, 30).Value =
                  this.tbDic.Text.Trim();
                fbCmd.Parameters.Add("@jmeno", FbDbType.VarChar, 30).Value =
                  this.tbJmeno.Text.Trim();
                fbCmd.Parameters.Add("@prijmeni", FbDbType.VarChar, 30).Value =
                  this.tbPrijmeni.Text.Trim();
                fbCmd.Parameters.Add("@ulice", FbDbType.VarChar, 100).Value =
                  this.tbUlice.Text.Trim();
                fbCmd.Parameters.Add("@mesto", FbDbType.VarChar, 30).Value =
                  this.tbMesto.Text.Trim();
                fbCmd.Parameters.Add("@psc", FbDbType.Integer).Value =
                  this.tbPsc.Text.Trim();
                fbCmd.Parameters.Add("@telefon", FbDbType.Integer).Value =
                  this.tbTelefon.Text.Trim();
                fbCmd.Parameters.Add("@email", FbDbType.VarChar, 30).Value =
                  this.tbEmail.Text.Trim();
                fbCmd.Parameters.Add("@kdozmenil", FbDbType.VarChar, 200).Value =
                  this.util.KdoZmenil;
                fbCmd.ExecuteNonQuery();

                iErrKod = 6108;
                if (this.bInsert)
                {
                    iErrKod = 6109;
                    this.iId = (int)fbParOut.Value;
                }

                iErrKod = 6110;
                fbCmd.Transaction.Commit();

                this.bZmena = this.Edit = false;
                this.NactiData();
                return true;
            }
            catch (Exception exc)
            {
                this.util.PisLog("Dodavatele " + iErrKod.ToString() + ". " +
                  exc.ToString(), true);
                return false;
            }
            finally
            {
                this.util.OdpojDb(fbCmd);
            }
        }

        private void btnZrusit_Click(object sender, EventArgs e)
        {
            int iErrKod = 6200;
            FbCommand fbCmd = null;
            string sErr;

            try
            {
                if (!this.util.ClsUziv.bStanice && !this.util.ClsUziv.bAdmin)
                {
                    this.util.PisLog("Rušit Dodavatele může administrátor, nebo správa stanice.", true);
                    return;
                }

                iErrKod = 6201;
                if (this.util.PisLog("Opravdu chcete zrušit Dodavatele \"" +
                  this.tbNazev.Text + "\"?", true, MessageBoxButtons.YesNo,
                  MessageBoxIcon.Question) != DialogResult.Yes)
                    return;

                iErrKod = 6202;
                if (!this.util.PripojDb(out fbCmd, out sErr))
                {
                    iErrKod = 6203;
                    this.util.PisLog("Dodavatele " + iErrKod.ToString() + ": " + sErr,
                      true);
                    return;
                }

                fbCmd.CommandText = "update tdodavatele set b_zrus = 'A', " +
                  "kdozmenil = @kdozmenil where id = @id";
                fbCmd.Parameters.Add("@kdozmenil", FbDbType.VarChar, 2000).Value =
                  this.util.KdoZmenil;
                fbCmd.Parameters.Add("@id", FbDbType.Integer).Value = this.iId;
                fbCmd.ExecuteNonQuery();
                fbCmd.Transaction.Commit();

                // Po zrušení uživatele bych se chtěl nastavit na následující nebo
                // předcházející řádek v gridu.
                if (this.gr.RowCount > this.gr.CurrentRow.Index + 1)
                    this.iId = (int)this.gr["id", this.gr.CurrentRow.Index + 1].Value;
                else
                if (this.gr.CurrentRow.Index > 0)
                    this.iId = (int)this.gr["id", this.gr.CurrentRow.Index - 1].Value;

                this.NactiData();
            }
            catch (Exception exc)
            {
                this.util.PisLog("Dodavatel " + iErrKod.ToString() + ": " +
                  exc.ToString(), true);
            }
            finally
            {
                this.util.OdpojDb(fbCmd);
            }
        }

        private void btnStorno_Click(object sender, EventArgs e)
        {
            this.bZmena = this.Edit = false;
            // Změním index řádku, aby se do detailu načetla aktuální věta z gridu.
            this.iRowInd = -2;
            this.ZmenaRadku();
        }

        private void gr_CurrentCellChanged(object sender, EventArgs e)
        {
            this.ZmenaRadku();
        }

        /// <summary>
        /// Obsluha změny aktuálního řádku v gridu.
        /// </summary>
        private void ZmenaRadku()
        {
            // Změnu budu evidovat jen v případě, že opravdu dojde ke změně řádku.
            if (this.bBlokZmenaRadku || this.Edit || this.gr.CurrentRow == null ||
              this.gr.CurrentRow.Index == this.iRowInd)
                return;

            this.iRowInd = this.gr.CurrentRow.Index;
            this.iId = (int)this.gr["id", iRowInd].Value;

            if (!this.Edit)
            {
                this.tbNazev.Text = (string)this.gr.CurrentRow.Cells["nazev"].Value;
                this.tbIco.Text =  Convert.ToString(this.gr.CurrentRow.Cells["ico"].Value);
                this.tbDic.Text = Convert.ToString(this.gr.CurrentRow.Cells["dic"].Value);
                this.tbPrijmeni.Text = (string)this.gr.CurrentRow.Cells[
                  "prijmeni"].Value;
                this.tbJmeno.Text = (string)this.gr.CurrentRow.Cells["jmeno"].Value;
                this.tbUlice.Text = (string)this.gr.CurrentRow.Cells[
                  "ulice"].Value;
                this.tbMesto.Text = (string)this.gr.CurrentRow.Cells[
                  "mesto"].Value;
                this.tbPsc.Text = Convert.ToString(this.gr.CurrentRow.Cells["psc"].Value);
                this.tbTelefon.Text = Convert.ToString(this.gr.CurrentRow.Cells["telefon"].Value);
                this.tbEmail.Text = (string)this.gr.CurrentRow.Cells["email"].Value;
            }
        }
    }
}
