﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using FirebirdSql.Data.FirebirdClient;
using System.IO;

namespace SkladEvidence
{
    /// <summary>
    /// Nastavení programu a údaje pro připojení k databázi.
    /// Číslování chyb: 3000 - 3999;
    /// </summary>
    internal partial class Nastaveni : Form, IForm
    {
        bool bKonec = false, bZmenaDtb = false, bBlokZmenyDtb = false;
        private Util util;

        //Parametry jsou uložené v xml souboru SkladEvidence.dtb, který je ve stejném adresáři jako spouštěný program.
        private string sSoubDtb = Application.StartupPath + "\\SkladEvidence.dtb";

        public Nastaveni(Util util)
        {
            InitializeComponent();
            this.util = util;
            this.NactiDtb();
        }

        public bool LzeUkoncit
        {
            get
            {
                return this.KontrolaUlozeni();
            }
        }

        public bool Konec
        {
            set
            {
                bKonec = value;
            }
        }

        /// <summary>
        /// Zkontroluje, zda jsou uložená všechny editovaná data. Pokud jsou uložená nebo uživatel nechce uložit, vrátí true.
        /// </summary>
        /// <returns></returns>
        private bool KontrolaUlozeni()
        {
            if (!this.util.ClsUziv.bAdmin)
                return true;

            if (this.bZmenaDtb)
            {
                switch (this.util.PisLog("Nastaveni: V datech pro připojení " +
                "k databázi jsou neuložené změny. Chcete data uložit?", true,
                MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question))
                {
                    case DialogResult.Yes:
                        if (!this.UlozDtb())
                            return false;
                        break;

                    case DialogResult.Cancel:
                        return false;

                    default:
                        this.bZmenaDtb = false;
                        return true;
                }
            }
            return true;
        }

        private void Nastaveni_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.Cancel = !this.bKonec && !this.Modal)
                this.Hide();
        }

        private void btnZobrazit_MouseDown(object sender, MouseEventArgs e)
        {
            this.tbHeslo.UseSystemPasswordChar = false;
        }

        private void btnZobrazit_MouseUp(object sender, MouseEventArgs e)
        {
            this.tbHeslo.UseSystemPasswordChar = true;
        }

        private void tbServer_TextChanged(object sender, EventArgs e)
        {
            if (!this.bBlokZmenyDtb)
                this.bZmenaDtb = true;
        }

        internal string FbConStr
        {
            get
            {
                FbConnectionStringBuilder fbSb = new FbConnectionStringBuilder();
                fbSb.DataSource = this.tbServer.Text;
                fbSb.Database = this.tbCestaKDatabazi.Text;
                fbSb.UserID = this.tbUzivatel.Text;
                fbSb.Password = this.tbHeslo.Text;
                return fbSb.ToString();
            }
        }

        private void btnUlozit_Click(object sender, EventArgs e)
        {
            // Je-li formulář zobrazený modálně, slouží pouze k zadání parametrů
            // pro připojení k databázi. Parametry se uloží, jestliže se do programu
            // přihlásí administrátor. 
            if (this.Modal)
            {
                this.DialogResult = DialogResult.OK;
                return;
            }

            if (!this.util.ClsUziv.bAdmin)
            {
                this.util.PisLog("Nastaveni: Ukládat data může jen administrátor.",
                  true);

                return;
            }

            this.UlozDtb();
        }

        private void btnStorno_Click(object sender, EventArgs e)
        {
            this.NactiDtb();
        }

        private void btnDbPripojeni_Click(object sender, EventArgs e)
        {
            FbCommand fbCmd = null;
            string sErr;

            try
            {
                this.util.FbConStr = this.FbConStr;

                if (this.util.PripojDb(out fbCmd, out sErr))
                {
                    this.util.PisLog("Připojení k databázi se podařilo.", true,
                      MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    this.util.PisLog("Při připojování k databázi došlo k chybě: " + sErr,
                      true);
                }
            }
            catch (Exception exc)
            {
                this.util.PisLog("Při připojování k databázi došlo k chybě: " +
                  exc.ToString(), true);
            }
            finally
            {
                this.util.OdpojDb(fbCmd);
            }
        }

        private void Nastaveni_Shown(object sender, EventArgs e)
        {
            if (this.Modal)
                this.btnUlozit.Text = "OK";
        }

        /// <summary>
        /// Načte parametry pro připojení k databázi.
        /// </summary>
        private void NactiDtb()
        {
            int iErrKod = 3100;

            try
            {
                if (!File.Exists(this.sSoubDtb))
                    return;

                iErrKod = 3101;
                this.ds.Clear();

                this.ds.ReadXml(this.sSoubDtb);

                iErrKod = 3102;
                if (!this.ds.Tables.Contains("dtb") || this.ds.Tables["dtb"].Rows.Count != 1)
                    return;

                this.bBlokZmenyDtb = true;

                this.tbServer.Text = (string)this.ds.Tables["dtb"].Rows[0]["server"];
                this.tbCestaKDatabazi.Text = (string)this.ds.Tables["dtb"].Rows[0]["cesta"];
                this.tbUzivatel.Text = (string)this.ds.Tables["dtb"].Rows[0]["uziv"];
                this.tbHeslo.Text = this.util.Desifr((string)this.ds.Tables["dtb"].Rows[0]["heslo"]);
            }
            catch (Exception exc)
            {
                this.util.PisLog("Nastaveni - NactiDtb " + iErrKod.ToString() +
                ": Při načítání parametrů pro připojení k databázi ze souboru \"" +
                this.sSoubDtb + "\" došlo k chybě: " + exc.ToString(), true);
            }
            finally
            {
                this.bBlokZmenyDtb = false;
            }
        }

        internal bool UlozDtb()
        {
            int iErrKod = 3200;
            DataRow row;

            try
            {
                this.ds.Tables["dtb"].Clear();
                iErrKod = 3201;
                row = this.ds.Tables["dtb"].NewRow();
                row["server"] = this.tbServer.Text;
                row["cesta"] = this.tbCestaKDatabazi.Text;
                row["uziv"] = this.tbUzivatel.Text;
                row["heslo"] = this.util.Sifr(this.tbHeslo.Text);
                this.ds.Tables["dtb"].Rows.Add(row);

                iErrKod = 30202;
                this.ds.WriteXml(this.sSoubDtb);

                this.bZmenaDtb = false;
                return true;
            }
            catch (Exception exc)
            {
                this.util.PisLog("Nastaveni - UlozDtb " + iErrKod.ToString() +
                ": Při ukládání parametrů pro připojení k databázi do souboru \"" +
                this.sSoubDtb + "\" došlo k chybě: " + exc.ToString(), true);

                return false;
            }
        }
    }
}
