﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using FirebirdSql.Data.FirebirdClient;

namespace SkladEvidence
{
    /// <summary>
    /// Správa uživatelů.
    /// Číslování chyb: 5000 - 5999.
    /// </summary>
    internal partial class Uzivatele : Form, IForm
    {
        private bool bKonec = false, bHistorie = false, bEdit = false,
        bInsert = false, bBlokZmenaRadku = false, bEdAdmin = false,
        bZmena = false;
        private int iId = -1, iRowInd = -1;
        private Util util;
        private string sEdHeslo;

        public Uzivatele(Util pUtil)
        {
            InitializeComponent();

            this.util = pUtil;
            this.Edit = this.Historie = false;
        }

        public bool LzeUkoncit
        {
            get
            {
                if (this.Edit)
                {
                    switch (this.util.PisLog("Uzivatele: Ve formuláři jsou " +
                      "rozpracovaná data. Chcete data uložit?", true,
                      MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question))
                    {
                        case DialogResult.Yes:
                            if (!this.UlozData())
                                return false;
                            break;

                        case DialogResult.No:
                            this.Edit = false;
                            break;

                        default:
                            return false;
                    }
                }

                return true;
            }
        }

        public bool Konec
        {
            set
            {
                this.bKonec = value;
            }
        }

        private void Uzivatele_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.Cancel = !this.bKonec)
                this.Hide();
        }

        private void Uzivatele_Load(object sender, EventArgs e)
        {
            // Umístím tlačítka na správná místa.
            this.btnOK.Location = this.btnZavritHist.Location =
              this.btnVlozit.Location;
            this.btnStorno.Location = this.btnUpravit.Location;
            this.NactiData();
        }

        private bool Edit
        {
            get
            {
                return this.bEdit;
            }
            set
            {
                this.bEdit = this.labHeslo.Visible = this.labHesloZnovu.Visible =
                  this.btnZobrazit.Visible = this.tbHeslo.Visible =
                  this.tbHesloZnovu.Visible = this.btnOK.Visible =
                  this.btnStorno.Visible = value;

                this.chbAdmin.ForeColor = this.chbStanice.ForeColor = Color.Black;

                this.btnHistorie.Visible = this.btnObnovit.Visible =
                  this.chbZrusene.Visible = this.tbLogin.ReadOnly =
                  this.tbPrijmeni.ReadOnly = this.tbJmeno.ReadOnly =
                  this.tbTitulPredJm.ReadOnly = this.tbTitulZaJm.ReadOnly = !value;

                this.btnVlozit.Visible = this.btnUpravit.Visible =
                this.btnZrusit.Visible = !this.chbZrusene.Checked && !value;
            }
        }

        private bool Historie
        {
            get
            {
                return this.bHistorie;
            }
            set
            {
                this.bHistorie = this.btnZavritHist.Visible = value;

                this.btnVlozit.Visible = this.btnUpravit.Visible =
                  this.btnZrusit.Visible = !this.chbZrusene.Checked && !value;

                this.btnHistorie.Visible = this.chbZrusene.Visible = !value;
            }
        }

        private bool NactiData()
        {
            int iErrKod = 50000;
            FbCommand fbCmd = null;
            FbDataAdapter fbAda = null;
            string sErr, sPole;
            int i1, iRInd, iCInd;

            try
            {
                // Po dobu načítání dat se nebude evidovat změna aktuálního řádku gridu.
                this.bBlokZmenaRadku = true;

                if (!this.util.PripojDb(out fbCmd, out sErr))
                {
                    iErrKod = 5001;
                    this.util.PisLog("Uzivatele " + iErrKod.ToString() + ": " + sErr,
                      true);
                    return false;
                }

                iRInd = (this.gr?.FirstDisplayedCell?.RowIndex ?? -1);

                iCInd = (this.gr?.FirstDisplayedCell?.ColumnIndex ?? -1);
                fbCmd.Parameters.Clear();
                this.tabUziv.Clear();

                sPole = "KDYZMENIL, ID, LOGIN, HESLO, PRIJMENI, JMENO, TITULPREDJM, " +
                  "TITULZAJM, B_ADMIN, B_STANICE, KDOZMENIL ";

                iErrKod = 5002;
                if (!this.Historie)
                {
                    if (!this.chbZrusene.Checked)
                    {
                        iErrKod = 5003;
                        // Všechny záznamy z tabulky TUZIVATELE.
                        fbCmd.CommandText = "select " + sPole +
                          "from tuzivatele order by login";
                    }
                    else
                    {
                        iErrKod = 5004;
                        /*
                        // Zrušené záznamy (nejsou v tabulce TUZIVATELE). Vyberu záznamy
                        // s nejnovějším datem změny, tj. datem zrušení.
                        fbCmd.CommandText = "select " + sPole +
                          "from tuzivatelehis uh where not exists (select 1 " +
                          "from tuzivatele u where uh.id = u.id) and " +
                          "uh.kdyzmenil = (select max(kdyzmenil) from tuzivatelehis uh1 " +
                          "where uh.id = uh1.id)";*/

                        // Zrušené záznamy mají v historické tabulce B_ZRUS = 'A'.
                        fbCmd.CommandText = "select " + sPole +
                          "from tuzivatelehis uh where b_zrus = 'A'";
                    }
                }
                else
                {
                    iErrKod = 5005;
                    // Záznamy z tabulek TUZIVATELE a TUZIVATELEHIS týkající se daného
                    // id.
                    fbCmd.CommandText = "select " + sPole +
                      "from tuzivatele where id = @id " +
                      "union select " + sPole +
                      "from tuzivatelehis where id = @id " +
                      "order by 1 desc";
                    fbCmd.Parameters.Add("@id", FbDbType.Integer).Value = this.iId;
                }

                fbAda = new FbDataAdapter(fbCmd);
                fbAda.Fill(this.tabUziv);

                // Pokusím se nastavit na stejné místo v gridu jako před načítáním dat.
                if (iRInd >= 0 && iCInd >= 0 && iRInd < this.gr.RowCount &&
                  iCInd < this.gr.ColumnCount)
                    this.gr.FirstDisplayedCell = this.gr[iCInd, iRInd];

                if (this.iId > 0)
                {
                    for (i1 = 0; i1 < this.gr.RowCount &&
                      (int)this.gr["id", i1].Value != this.iId; ++i1) ;

                    if (i1 < this.gr.RowCount)
                        this.gr.CurrentCell =
                          this.gr[(iCInd >= 0 && iCInd < this.gr.ColumnCount ? iCInd : 0),
                          i1];
                }

                return true;
            }
            catch (Exception exc)
            {
                this.util.PisLog("Uzivatele " + iErrKod.ToString() + ": " +
                  exc.ToString(), true);

                return false;
            }
            finally
            {
                this.bBlokZmenaRadku = false;
                // Musím zajistit aktualizaci iId.
                this.iRowInd = -1;
                this.ZmenaRadku();

                if (fbAda != null)
                    fbAda.Dispose();

                this.util.OdpojDb(fbCmd);
            }
        }

        private void Uzivatele_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.F5)
                this.NactiData();
        }

        private void btnObnovit_Click(object sender, EventArgs e)
        {
            this.NactiData();
        }

        private void chbZrusene_CheckedChanged(object sender, EventArgs e)
        {
            this.btnVlozit.Visible = this.btnUpravit.Visible =
              this.btnZrusit.Visible = !this.chbZrusene.Checked;
            this.NactiData();
        }

        private void btnHistorie_Click(object sender, EventArgs e)
        {
            this.Historie = true;
            this.NactiData();
        }

        private void btnZavritHist_Click(object sender, EventArgs e)
        {
            this.Historie = false;
            this.NactiData();
        }

        private void btnVlozit_Click(object sender, EventArgs e)
        {
            if (!this.util.ClsUziv.bAdmin)
            {
                this.util.PisLog("Vkládat uživatele může jen administrátor.", true);
                return;
            }

            this.bEdAdmin = false;
            this.Edit = this.bInsert = true;

            this.tbLogin.Text = this.tbPrijmeni.Text = this.tbJmeno.Text =
              this.tbTitulPredJm.Text = this.tbTitulZaJm.Text = this.tbHeslo.Text =
              this.tbHesloZnovu.Text = "";
            this.chbAdmin.Checked = this.chbStanice.Checked = false;
        }

        private void btnUpravit_Click(object sender, EventArgs e)
        {
            if (!this.util.ClsUziv.bAdmin)
            {
                this.util.PisLog("Editovat uživatele může jen administrátor.", true);
                return;
            }

            this.bInsert = false;

            // Pro editace admina platí odlišná pravidla, proto nahodím příznak, když 
            // se edituje admin.
            this.bEdAdmin = this.tbLogin.Text.ToUpper() == "ADMIN";
            this.Edit = true;
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            this.UlozData();
        }

        private bool UlozData()
        {
            int iErrKod = 5100;
            int i1;
            string sLogin, sErr;
            FbCommand fbCmd = null;
            FbParameter fbParOut = null;

            try
            {
                if ((sLogin = this.tbLogin.Text.Trim().ToUpper()) == "")
                {
                    this.util.PisLog("Login musí být zadaný.", true);
                    this.tbLogin.Focus();
                    return false;
                }

                iErrKod = 5101;
                if (this.bEdAdmin && sLogin != "ADMIN")
                {
                    this.util.PisLog("Uživateli \"ADMIN\" musí zůstat login \"ADMIN\" " +
                      "(na velikosti písmen nezáleží).", true);
                    this.tbLogin.Focus();
                    return false;
                }

                // Toto by nastat nemělo, protože je to hlídané na události Click, ale co 
                // kdyby ...
                iErrKod = 5102;
                if (this.bEdAdmin && !this.chbAdmin.Checked)
                    this.chbAdmin.Checked = true;

                for (i1 = 0; i1 < this.tabUziv.Rows.Count; ++i1)
                {
                    if (sLogin == ((string)this.tabUziv.Rows[i1]["login"]).Trim().
                      ToUpper() && (this.bInsert || this.iId !=
                      (int)this.tabUziv.Rows[i1]["id"]))
                    {
                        this.util.PisLog("Login \"" + sLogin + "\" je již použitý " +
                          "u uživatele \"" + (string)this.tabUziv.Rows[i1]["prijmeni"] +
                          " " + (string)this.tabUziv.Rows[i1]["jmeno"] + "\".", true);
                        this.tbLogin.Focus();
                        return false;
                    }
                }

                iErrKod = 5103;
                if (this.bInsert && this.tbHeslo.Text == "")
                {
                    this.util.PisLog("Heslo musí být zadané.", true);
                    this.tbHeslo.Focus();
                    return false;
                }

                if (this.tbHeslo.Text != this.tbHesloZnovu.Text)
                {
                    this.util.PisLog("Nesouhlasí heslo a opakované heslo.", true);
                    this.tbHeslo.Focus();
                    return false;
                }

                if (this.tbHeslo.Text != "")
                    this.sEdHeslo = this.util.Sifr(this.tbHeslo.Text);

                iErrKod = 5104;
                if (!this.util.PripojDb(out fbCmd, out sErr))
                {
                    iErrKod = 5105;
                    this.util.PisLog("Uzivatele " + iErrKod.ToString() + ": " + sErr,
                      true);
                    return false;
                }

                if (this.bInsert)
                {
                    iErrKod = 5106;
                    fbCmd.CommandText = "insert into tuzivatele (LOGIN, HESLO, " +
                      "PRIJMENI, JMENO, TITULPREDJM, TITULZAJM, B_ADMIN, B_STANICE, " +
                      "KDOZMENIL) " +
                      "values (@LOGIN, @HESLO, @PRIJMENI, @JMENO, @TITULPREDJM, " +
                      "@TITULZAJM, @B_ADMIN, @B_STANICE, @KDOZMENIL) " +
                      "returning id";
                    // Parametr pro vrácení id vložené věty.
                    fbParOut = new FbParameter();
                    fbParOut.Direction = ParameterDirection.Output;
                    fbParOut.DbType = DbType.Int32;
                    fbParOut.SourceColumn = "id";
                    fbCmd.Parameters.Add(fbParOut);
                }
                else
                {
                    iErrKod = 5107;
                    fbCmd.CommandText = "update tuzivatele set LOGIN = @login, " +
                      "HESLO = @heslo, PRIJMENI = @prijmeni, JMENO = @jmeno, " +
                      "TITULPREDJM = @titulpredjm, TITULZAJM = @titulzajm, " +
                      "B_ADMIN = @b_admin, B_STANICE = @b_stanice, " +
                      "KDOZMENIL = @kdozmenil " +
                      "where id = @id";
                    fbCmd.Parameters.Add("@id", FbDbType.Integer).Value = this.iId;
                }

                fbCmd.Parameters.Add("@login", FbDbType.VarChar, 30).Value =
                  this.tbLogin.Text.Trim();
                fbCmd.Parameters.Add("@heslo", FbDbType.VarChar, 200).Value =
                  this.sEdHeslo;
                fbCmd.Parameters.Add("@prijmeni", FbDbType.VarChar, 30).Value =
                  this.tbPrijmeni.Text.Trim();
                fbCmd.Parameters.Add("@jmeno", FbDbType.VarChar, 30).Value =
                  this.tbJmeno.Text.Trim();
                fbCmd.Parameters.Add("@titulpredjm", FbDbType.VarChar, 30).Value =
                  this.tbTitulPredJm.Text.Trim();
                fbCmd.Parameters.Add("@titulzajm", FbDbType.VarChar, 30).Value =
                  this.tbTitulZaJm.Text.Trim();
                fbCmd.Parameters.Add("@b_admin", FbDbType.Char, 1).Value =
                  this.chbAdmin.Checked ? "A" : "N";
                fbCmd.Parameters.Add("@b_stanice", FbDbType.Char, 1).Value =
                  this.chbStanice.Checked ? "A" : "N";
                fbCmd.Parameters.Add("@kdozmenil", FbDbType.VarChar, 200).Value =
                  this.util.KdoZmenil;
                fbCmd.ExecuteNonQuery();

                iErrKod = 5108;
                if (this.bInsert)
                {
                    iErrKod = 5109;
                    this.iId = (int)fbParOut.Value;
                }

                iErrKod = 5110;
                fbCmd.Transaction.Commit();

                this.bZmena = this.Edit = false;
                this.tbHeslo.Text = this.tbHesloZnovu.Text = "";
                this.NactiData();
                return true;
            }
            catch (Exception exc)
            {
                this.util.PisLog("Uzivatele " + iErrKod.ToString() + ". " +
                  exc.ToString(), true);
                return false;
            }
            finally
            {
                this.util.OdpojDb(fbCmd);
            }
        }

        private void tb_TextChanged(object sender, EventArgs e)
        {
            if (this.Edit)
                this.bZmena = true;
        }

        private void btnZrusit_Click(object sender, EventArgs e)
        {
            int iErrKod = 5200;
            FbCommand fbCmd = null;
            string sErr;

            try
            {
                if (!this.util.ClsUziv.bAdmin)
                {
                    this.util.PisLog("Rušit uživatele může jen administrátor.", true);
                    return;
                }

                iErrKod = 5201;
                if (this.tbLogin.Text.ToUpper() == "ADMIN")
                {
                    this.util.PisLog("Uživatele \"ADMIN\" nelze zrušit.", true);
                    return;
                }

                iErrKod = 5202;
                if (this.tbLogin.Text.ToUpper() == this.util.ClsUziv.sLogin.ToUpper())
                {
                    this.util.PisLog("Nelze zrušit uživatele, kterým jste přihlášení " +
                      "do programu.", true);
                    return;
                }

                iErrKod = 5203;
                if (this.util.PisLog("Opravdu chcete zrušit uživatele \"" +
                  this.tbLogin.Text + " - " + this.tbPrijmeni.Text + " " +
                  this.tbJmeno.Text + "\"?", true, MessageBoxButtons.YesNo,
                  MessageBoxIcon.Question) != DialogResult.Yes)
                    return;

                iErrKod = 5204;
                if (!this.util.PripojDb(out fbCmd, out sErr))
                {
                    iErrKod = 5205;
                    this.util.PisLog("Uzivatele " + iErrKod.ToString() + ": " + sErr,
                      true);
                    return;
                }

                fbCmd.CommandText = "update tuzivatele set b_zrus = 'A', " +
                  "kdozmenil = @kdozmenil where id = @id";
                fbCmd.Parameters.Add("@kdozmenil", FbDbType.VarChar, 2000).Value =
                  this.util.KdoZmenil;
                fbCmd.Parameters.Add("@id", FbDbType.Integer).Value = this.iId;
                fbCmd.ExecuteNonQuery();
                fbCmd.Transaction.Commit();

                // Po zrušení uživatele bych se chtěl nastavit na následující nebo
                // předcházející řádek v gridu.
                if (this.gr.RowCount > this.gr.CurrentRow.Index + 1)
                    this.iId = (int)this.gr["id", this.gr.CurrentRow.Index + 1].Value;
                else
                if (this.gr.CurrentRow.Index > 0)
                    this.iId = (int)this.gr["id", this.gr.CurrentRow.Index - 1].Value;

                this.NactiData();
            }
            catch (Exception exc)
            {
                this.util.PisLog("Uzivatele " + iErrKod.ToString() + ": " +
                  exc.ToString(), true);
            }
            finally
            {
                this.util.OdpojDb(fbCmd);
            }
        }

        private void chb_Click(object sender, EventArgs e)
        {
            // Když není režim editace, vrátím zpátky původní hodnotu.
            if (!this.Edit)
                ((CheckBox)sender).Checked = !((CheckBox)sender).Checked;
        }

        private void chbAdmin_CheckedChanged(object sender, EventArgs e)
        {
            if (this.Edit && !this.chbAdmin.Checked && this.bEdAdmin)
            {
                this.util.PisLog("Uživateli \"ADMIN\" nelze odebrat administrátorská " +
                  "práva.", true);

                this.chbAdmin.Checked = true;
            }
        }

        private void btnStorno_Click(object sender, EventArgs e)
        {
            this.bZmena = this.Edit = false;
            // Změním index řádku, aby se do detailu načetla aktuální věta z gridu.
            this.iRowInd = -2;
            this.ZmenaRadku();
        }

        private void gr_CurrentCellChanged(object sender, EventArgs e)
        {
            this.ZmenaRadku();
        }

        private void btnZobrazit_MouseDown(object sender, MouseEventArgs e)
        {
            this.tbHeslo.UseSystemPasswordChar = false;
        }

        private void btnZobrazit_MouseUp(object sender, MouseEventArgs e)
        {
            this.tbHeslo.UseSystemPasswordChar = true;
        }

        /// <summary>
        /// Obsluha změny aktuálního řádku v gridu.
        /// </summary>
        private void ZmenaRadku()
        {
            // Změnu budu evidovat jen v případě, že opravdu dojde ke změně řádku.
            if (this.bBlokZmenaRadku || this.Edit || this.gr.CurrentRow == null ||
              this.gr.CurrentRow.Index == this.iRowInd)
                return;

            this.iRowInd = this.gr.CurrentRow.Index;
            this.iId = (int)this.gr["id", iRowInd].Value;

            if (!this.Edit)
            {
                this.tbLogin.Text = (string)this.gr.CurrentRow.Cells["login"].Value;
                this.tbPrijmeni.Text = (string)this.gr.CurrentRow.Cells[
                  "prijmeni"].Value;
                this.tbJmeno.Text = (string)this.gr.CurrentRow.Cells["jmeno"].Value;
                this.tbTitulPredJm.Text = (string)this.gr.CurrentRow.Cells[
                  "TitulPredJm"].Value;
                this.tbTitulZaJm.Text = (string)this.gr.CurrentRow.Cells[
                  "TitulZaJm"].Value;
                this.sEdHeslo = (string)this.gr.CurrentRow.Cells["heslo"].Value;
                this.chbAdmin.Checked = (bool)this.gr.CurrentRow.Cells["admin"].Value;
                this.chbStanice.Checked = (bool)this.gr.CurrentRow.Cells[
                  "stanice"].Value;
            }
        }
    }
}
