﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using FirebirdSql.Data.FirebirdClient;

namespace SkladEvidence
{
    /// <summary>
    /// Správa uživatelů.
    /// Číslování chyb: 7000 - 7999.
    /// </summary>
    /// 
    internal partial class Zakaznici : Form, IForm
    {
        private bool bKonec = false, bHistorie = false, bEdit = false,
        bInsert = false, bBlokZmenaRadku = false, bEdAdmin = false,
        bZmena = false;
        private char sOsoba = 'P';
        private int iId = -1, iRowInd = -1;
        private Util util;
        private EnZakaznici enZak;

        public Zakaznici(Util pUtil, EnZakaznici penZak)
        {
            InitializeComponent();
            this.util = pUtil;
            this.Edit = this.Historie = false;
            this.enZak = penZak;

            switch (this.enZak)
            {
                case EnZakaznici.fyzOsoby:
                    this.Text = "Fyzické osoby";
                    this.sOsoba = 'F';
                    break;

                case EnZakaznici.pravOsoby:
                    this.Text = "Pravnické osoby";
                    this.sOsoba = 'P';
                    break;
            }
        }

        public bool LzeUkoncit
        {
            get
            {
                if (this.Edit)
                {
                    switch (this.util.PisLog("Zákazníci: Ve formuláři jsou " +
                      "rozpracovaná data. Chcete data uložit?", true,
                      MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question))
                    {
                        case DialogResult.Yes:
                            if (!this.UlozData())
                                return false;
                            break;

                        case DialogResult.No:
                            this.Edit = false;
                            break;

                        default:
                            return false;
                    }
                }

                return true;
            }
        }

        public bool Konec
        {
            set
            {
                bKonec = value;
            }
        }

        private void Zakaznici_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.Cancel = !this.bKonec)
                this.Hide();
        }

        private void Zakaznici_Load(object sender, EventArgs e)
        {
            // Umístím tlačítka na správná místa.
            this.btnOK.Location = this.btnZavritHist.Location =
              this.btnVlozit.Location;
            this.btnStorno.Location = this.btnUpravit.Location;

            if (this.sOsoba == 'F')
            {
                this.tbNazev.Visible = this.tbDic.Visible = this.tbIco.Visible =
                this.lbNazev.Visible = this.lbDic.Visible = this.lbIco.Visible = false; 
            }

            this.NactiData();
        }

        private bool Edit
        {
            get
            {
                return this.bEdit;
            }
            set
            {
                this.bEdit = this.btnOK.Visible = this.btnStorno.Visible = value;

                this.btnHistorie.Visible = this.btnObnovit.Visible =
                  this.chbZrusene.Visible = this.tbNazev.ReadOnly = this.tbIco.ReadOnly =
                  this.tbDic.ReadOnly = this.tbPrijmeni.ReadOnly = this.tbJmeno.ReadOnly =
                  this.tbUlice.ReadOnly = this.tbMesto.ReadOnly =
                  this.tbPsc.ReadOnly = this.tbTelefon.ReadOnly = this.tbEmail.ReadOnly = !value;

                this.btnVlozit.Visible = this.btnUpravit.Visible =
                this.btnZrusit.Visible = !this.chbZrusene.Checked && !value;
            }
        }

        private bool Historie
        {
            get
            {
                return this.bHistorie;
            }
            set
            {
                this.bHistorie = this.btnZavritHist.Visible = value;

                this.btnVlozit.Visible = this.btnUpravit.Visible =
                  this.btnZrusit.Visible = !this.chbZrusene.Checked && !value;

                this.btnHistorie.Visible = this.chbZrusene.Visible = !value;
            }
        }

        private bool NactiData()
        {
            int iErrKod = 7000;
            FbCommand fbCmd = null;
            FbDataAdapter fbAda = null;
            string sErr, sPole;
            int i1, iRInd, iCInd;

            try
            {
                // Po dobu načítání dat se nebude evidovat změna aktuálního řádku gridu.
                this.bBlokZmenaRadku = true;

                if (!this.util.PripojDb(out fbCmd, out sErr))
                {
                    iErrKod = 7001;
                    this.util.PisLog("Uzivatele " + iErrKod.ToString() + ": " + sErr,
                      true);
                    return false;
                }

                iRInd = (this.gr?.FirstDisplayedCell?.RowIndex ?? -1);

                iCInd = (this.gr?.FirstDisplayedCell?.ColumnIndex ?? -1);
                fbCmd.Parameters.Clear();
                this.tabZak.Clear();

                sPole = "KDYZMENIL, ID, NAZEV, ICO, DIC, PRIJMENI, JMENO, ULICE, " +
                  "MESTO, PSC, TELEFON, EMAIL, KDOZMENIL ";

                iErrKod = 7002;
                if (!this.Historie)
                {
                    if (!this.chbZrusene.Checked)
                    {
                        iErrKod = 7003;
                        // Všechny záznamy z tabulky TZAKAZNICI.
                        fbCmd.CommandText = "select " + sPole +
                          "from tzakaznici where osoba = @osoba";
                        fbCmd.Parameters.Add("@osoba", FbDbType.Char).Value =
                        this.sOsoba;
                    }
                    else
                    {
                        iErrKod = 7004;
                        /*
                        // Zrušené záznamy (nejsou v tabulce TZAKAZNICI). Vyberu záznamy
                        // s nejnovějším datem změny, tj. datem zrušení.
                        fbCmd.CommandText = "select " + sPole +
                          "from tzakaznicihis uh where not exists (select 1 " +
                          "from tzakaznici u where uh.id = u.id) and " +
                          "uh.kdyzmenil = (select max(kdyzmenil) from tzakaznici uh1 " +
                          "where uh.id = uh1.id)";*/

                        // Zrušené záznamy mají v historické tabulce B_ZRUS = 'A'.
                        fbCmd.CommandText = "select " + sPole +
                          "from tzakaznicihis uh where b_zrus = 'A' and osoba = @osoba";
                        fbCmd.Parameters.Add("@osoba", FbDbType.Char).Value =
                        this.sOsoba;
                    }
                }
                else
                {
                    iErrKod = 7005;
                    // Záznamy z tabulek TZAKAZNICI a TZAKAZNICIHIS týkající se daného
                    // id.
                    fbCmd.CommandText = "select " + sPole +
                      "from tzakaznici where id = @id and osoba = @osoba " +
                      "union select " + sPole +
                      "from tzakaznicihis where id = @id and osoba = @osoba " +
                      "order by 1 desc";
                    fbCmd.Parameters.Add("@id", FbDbType.Integer).Value = this.iId;
                    fbCmd.Parameters.Add("@osoba", FbDbType.Char).Value = this.sOsoba;
                }

                fbAda = new FbDataAdapter(fbCmd);
                fbAda.Fill(this.tabZak);

                // Pokusím se nastavit na stejné místo v gridu jako před načítáním dat.
                if (iRInd >= 0 && iCInd >= 0 && iRInd < this.gr.RowCount &&
                  iCInd < this.gr.ColumnCount)
                    this.gr.FirstDisplayedCell = this.gr[iCInd, iRInd];

                if (this.iId > 0)
                {
                    for (i1 = 0; i1 < this.gr.RowCount &&
                      (int)this.gr["id", i1].Value != this.iId; ++i1) ;

                    if (i1 < this.gr.RowCount)
                        this.gr.CurrentCell =
                          this.gr[(iCInd >= 0 && iCInd < this.gr.ColumnCount ? iCInd : 0),
                          i1];
                }

                return true;
            }
            catch (Exception exc)
            {
                this.util.PisLog("Zákazníci " + iErrKod.ToString() + ": " +
                  exc.ToString(), true);

                return false;
            }
            finally
            {
                this.bBlokZmenaRadku = false;
                // Musím zajistit aktualizaci iId.
                this.iRowInd = -1;
                this.ZmenaRadku();

                if (fbAda != null)
                    fbAda.Dispose();

                if (sOsoba == 'F')
                {
                    this.nazev.Visible = this.ico.Visible = this.dic.Visible = false;
                }

                this.util.OdpojDb(fbCmd);
            }
        }

        private void Zakaznici_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.F5)
                this.NactiData();
        }

        private void btnObnovit_Click(object sender, EventArgs e)
        {
            this.NactiData();
        }

        private void chbZrusene_CheckedChanged(object sender, EventArgs e)
        {
            this.btnVlozit.Visible = this.btnUpravit.Visible =
              this.btnZrusit.Visible = !this.chbZrusene.Checked;
            this.NactiData();
        }

        private void btnHistorie_Click(object sender, EventArgs e)
        {
            this.Historie = true;
            this.NactiData();
        }

        private void btnZavritHist_Click(object sender, EventArgs e)
        {
            this.Historie = false;
            this.NactiData();
        }

        private void btnVlozit_Click(object sender, EventArgs e)
        {
            if (!this.util.ClsUziv.bStanice && !this.util.ClsUziv.bAdmin)
            {
                this.util.PisLog("Vkládat zákazníky může administrátor, nebo správa stanice.", true);
                return;
            }

            this.bEdAdmin = false;
            this.Edit = this.bInsert = true;

            this.tbNazev.Text = this.tbIco.Text = this.tbDic.Text = this.tbPrijmeni.Text = this.tbJmeno.Text =
              this.tbUlice.Text = this.tbMesto.Text = this.tbPsc.Text = this.tbTelefon.Text =
              this.tbEmail.Text = "";
        }

        private void btnUpravit_Click(object sender, EventArgs e)
        {
            if (!this.util.ClsUziv.bStanice && !this.util.ClsUziv.bAdmin)
            {
                this.util.PisLog("Editovat zákazníky může administrátor, nebo správa stanice.", true);
                return;
            }

            this.bInsert = false;

            this.Edit = true;
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            this.UlozData();
        }

        private bool UlozData()
        {
            int iErrKod = 7100;
            string sErr;
            FbCommand fbCmd = null;
            FbParameter fbParOut = null;

            try
            {
                iErrKod = 7101;
                if (this.sOsoba == 'P' && this.tbNazev.Text == "")
                {
                    this.util.PisLog("Název firmy musí být zadán.", true);
                    this.tbNazev.Focus();
                    return false;
                }

                if (this.sOsoba == 'F' && this.tbPrijmeni.Text == "")
                {
                    this.util.PisLog("Jméno osoby musí být zadáno.", true);
                    this.tbNazev.Focus();
                    return false;
                }

                if (this.tbTelefon.Text == "")
                {
                    this.util.PisLog("Telefon zákazníka musí být zadán.", true);
                    this.tbNazev.Focus();
                    return false;
                }

                if (this.tbPsc.Text == "")
                {
                    this.util.PisLog("PSČ zákazníka musí být zadáno.", true);
                    this.tbNazev.Focus();
                    return false;
                }

                iErrKod = 7102;
                if (!this.util.PripojDb(out fbCmd, out sErr))
                {
                    iErrKod = 7103;
                    this.util.PisLog("Zákazníci " + iErrKod.ToString() + ": " + sErr,
                      true);
                    return false;
                }

                if (this.bInsert)
                {
                    iErrKod = 7104;
                    fbCmd.CommandText = "insert into tzakaznici (NAZEV, ICO, " +
                      "DIC, PRIJMENI, JMENO, ULICE, MESTO, PSC, TELEFON, " +
                      "EMAIL, KDOZMENIL, OSOBA) " +
                      "values (@NAZEV, @ICO, @DIC, @PRIJMENI, @JMENO, @ULICE, " +
                      "@MESTO, @PSC, @TELEFON, @EMAIL, @KDOZMENIL, @OSOBA) " +
                      "returning id";
          
                    // Parametr pro vrácení id vložené věty.
                    fbParOut = new FbParameter();
                    fbParOut.Direction = ParameterDirection.Output;
                    fbParOut.DbType = DbType.Int32;
                    fbParOut.SourceColumn = "id";
                    fbCmd.Parameters.Add(fbParOut);
                    fbCmd.Parameters.Add("@nazev", FbDbType.VarChar, 30).Value =
                  this.tbNazev.Text.Trim();
                    fbCmd.Parameters.Add("@ico", FbDbType.VarChar, 30).Value =
                      this.tbIco.Text.Trim();
                    fbCmd.Parameters.Add("@dic", FbDbType.VarChar, 30).Value =
                      this.tbDic.Text.Trim();
                    fbCmd.Parameters.Add("@prijmeni", FbDbType.VarChar, 30).Value =
                      this.tbPrijmeni.Text.Trim();
                    fbCmd.Parameters.Add("@jmeno", FbDbType.VarChar, 30).Value =
                      this.tbJmeno.Text.Trim();
                    fbCmd.Parameters.Add("@ulice", FbDbType.VarChar, 100).Value =
                      this.tbUlice.Text.Trim();
                    fbCmd.Parameters.Add("@mesto", FbDbType.VarChar, 30).Value =
                      this.tbMesto.Text.Trim();
                    fbCmd.Parameters.Add("@psc", FbDbType.Integer).Value =
                      this.tbPsc.Text.Trim();
                    fbCmd.Parameters.Add("@telefon", FbDbType.Integer).Value =
                      this.tbTelefon.Text.Trim();
                    fbCmd.Parameters.Add("@email", FbDbType.VarChar, 30).Value =
                      this.tbEmail.Text.Trim();
                    fbCmd.Parameters.Add("@kdozmenil", FbDbType.VarChar, 200).Value =
                      this.util.KdoZmenil;
                    fbCmd.Parameters.Add("@osoba", FbDbType.Char).Value = this.sOsoba;
                }
                else
                {
                    iErrKod = 7105;
                    fbCmd.CommandText = "update tzakaznici set NAZEV = @nazev, " +
                      "ICO = @ico, DIC = @dic, PRIJMENI = @prijmeni, JMENO = @jmeno, " +
                      "ULICE = @ulice, MESTO = @mesto, " +
                      "PSC = @psc, TELEFON = @telefon, EMAIL = @email," +
                      "KDOZMENIL = @kdozmenil " +
                      "where id = @id";
                    fbCmd.Parameters.Add("@id", FbDbType.Integer).Value = this.iId;
                    fbCmd.Parameters.Add("@nazev", FbDbType.VarChar, 30).Value =
                  this.tbNazev.Text.Trim();
                    fbCmd.Parameters.Add("@ico", FbDbType.VarChar, 30).Value =
                      this.tbIco.Text.Trim();
                    fbCmd.Parameters.Add("@dic", FbDbType.VarChar, 30).Value =
                      this.tbDic.Text.Trim();
                    fbCmd.Parameters.Add("@prijmeni", FbDbType.VarChar, 30).Value =
                      this.tbPrijmeni.Text.Trim();
                    fbCmd.Parameters.Add("@jmeno", FbDbType.VarChar, 30).Value =
                      this.tbJmeno.Text.Trim();
                    fbCmd.Parameters.Add("@ulice", FbDbType.VarChar, 100).Value =
                      this.tbUlice.Text.Trim();
                    fbCmd.Parameters.Add("@mesto", FbDbType.VarChar, 30).Value =
                      this.tbMesto.Text.Trim();
                    fbCmd.Parameters.Add("@psc", FbDbType.Integer).Value =
                      this.tbPsc.Text.Trim();
                    fbCmd.Parameters.Add("@telefon", FbDbType.Integer).Value =
                      this.tbTelefon.Text.Trim();
                    fbCmd.Parameters.Add("@email", FbDbType.VarChar, 30).Value =
                      this.tbEmail.Text.Trim();
                    fbCmd.Parameters.Add("@kdozmenil", FbDbType.VarChar, 200).Value =
                      this.util.KdoZmenil;
                }
                
                fbCmd.ExecuteNonQuery();

                iErrKod = 7106;
                if (this.bInsert)
                {
                    iErrKod = 7107;
                    this.iId = (int)fbParOut.Value;
                }

                iErrKod = 7108;
                fbCmd.Transaction.Commit();

                this.bZmena = this.Edit = false;
                this.NactiData();
                return true;
            }
            catch (Exception exc)
            {
                this.util.PisLog("Zakaznici " + iErrKod.ToString() + ". " +
                  exc.ToString(), true);
                return false;
            }
            finally
            {
                this.util.OdpojDb(fbCmd);
            }
        }

        private void btnZrusit_Click(object sender, EventArgs e)
        {
            int iErrKod = 7200;
            FbCommand fbCmd = null;
            string sErr;

            try
            {
                if (!this.util.ClsUziv.bStanice && !this.util.ClsUziv.bAdmin)
                {
                    this.util.PisLog("Rušit zákazníky může administrátor, nebo správa stanice.", true);
                    return;
                }

                iErrKod = 7201;
                if (this.util.PisLog("Opravdu chcete zrušit Zákazníky \"" +
                  this.tbNazev.Text + "\"?", true, MessageBoxButtons.YesNo,
                  MessageBoxIcon.Question) != DialogResult.Yes)
                    return;

                iErrKod = 7202;
                if (!this.util.PripojDb(out fbCmd, out sErr))
                {
                    iErrKod = 7203;
                    this.util.PisLog("Zákazníci " + iErrKod.ToString() + ": " + sErr,
                      true);
                    return;
                }

                fbCmd.CommandText = "update tzakaznici set b_zrus = 'A', " +
                  "kdozmenil = @kdozmenil where id = @id";
                fbCmd.Parameters.Add("@kdozmenil", FbDbType.VarChar, 2000).Value =
                  this.util.KdoZmenil;
                fbCmd.Parameters.Add("@id", FbDbType.Integer).Value = this.iId;
                fbCmd.ExecuteNonQuery();
                fbCmd.Transaction.Commit();

                // Po zrušení uživatele bych se chtěl nastavit na následující nebo
                // předcházející řádek v gridu.
                if (this.gr.RowCount > this.gr.CurrentRow.Index + 1)
                    this.iId = (int)this.gr["id", this.gr.CurrentRow.Index + 1].Value;
                else
                if (this.gr.CurrentRow.Index > 0)
                    this.iId = (int)this.gr["id", this.gr.CurrentRow.Index - 1].Value;

                this.NactiData();
            }
            catch (Exception exc)
            {
                this.util.PisLog("Uzivatele " + iErrKod.ToString() + ": " +
                  exc.ToString(), true);
            }
            finally
            {
                this.util.OdpojDb(fbCmd);
            }
        }

        private void btnStorno_Click(object sender, EventArgs e)
        {
            this.bZmena = this.Edit = false;
            // Změním index řádku, aby se do detailu načetla aktuální věta z gridu.
            this.iRowInd = -2;
            this.ZmenaRadku();
        }

        private void gr_CurrentCellChanged(object sender, EventArgs e)
        {
            this.ZmenaRadku();
        }

        /// <summary>
        /// Obsluha změny aktuálního řádku v gridu.
        /// </summary>
        private void ZmenaRadku()
        {
            // Změnu budu evidovat jen v případě, že opravdu dojde ke změně řádku.
            if (this.bBlokZmenaRadku || this.Edit || this.gr.CurrentRow == null ||
              this.gr.CurrentRow.Index == this.iRowInd)
                return;

            this.iRowInd = this.gr.CurrentRow.Index;
            this.iId = (int)this.gr["id", iRowInd].Value;

            if (!this.Edit)
            {
                this.tbNazev.Text = (string)this.gr.CurrentRow.Cells["nazev"].Value;
                this.tbIco.Text = Convert.ToString(this.gr.CurrentRow.Cells["ico"].Value);
                this.tbDic.Text = Convert.ToString(this.gr.CurrentRow.Cells["dic"].Value);
                this.tbPrijmeni.Text = (string)this.gr.CurrentRow.Cells[
                  "prijmeni"].Value;
                this.tbJmeno.Text = (string)this.gr.CurrentRow.Cells["jmeno"].Value;
                this.tbUlice.Text = (string)this.gr.CurrentRow.Cells[
                  "ulice"].Value;
                this.tbMesto.Text = (string)this.gr.CurrentRow.Cells[
                  "mesto"].Value;
                this.tbPsc.Text = Convert.ToString(this.gr.CurrentRow.Cells["psc"].Value);
                this.tbTelefon.Text = Convert.ToString(this.gr.CurrentRow.Cells["telefon"].Value);
                this.tbEmail.Text = (string)this.gr.CurrentRow.Cells["email"].Value;
            }
        }
    }
}
