﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using FirebirdSql.Data.FirebirdClient;
using System.IO;

namespace SkladEvidence
{
    /// <summary>
    /// Správa uživatelů.
    /// Číslování chyb: 8000 - 8999.
    /// </summary>
    internal partial class Zbozi : Form, IForm
    {
        private bool bKonec = false, bHistorie = false, bEdit = false,
        bInsert = false, bBlokZmenaRadku = false, bEdAdmin = false,
        bZmena = false;
        private string sSoub = "";
        private int iId = -1, iRowInd = -1;
        private Encoding enc = null;
        private StringBuilder sbErr = new StringBuilder();
        private Util util;

        public Zbozi(Util pUtil)
        {
            InitializeComponent();
            this.util = pUtil;
            this.Edit = this.Historie = false;
            this.enc = Encoding.GetEncoding(1250);
        }

        public bool LzeUkoncit
        {
            get
            {
                // Pokud se právě nahrává číselník, zeptám se, zda mám nahrávání 
                // ukončit.
                return !this.bw.IsBusy || this.util.PisLog("Právě se nahrává zboží. " +
                  "Má se nahrávání ukončit?",
                  true, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) ==
                  DialogResult.Yes;
            }
        }

        public bool Konec
        {
            set
            {
                bKonec = value;
            }
        }

        private void Zbozí_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.Cancel = !this.bKonec)
                this.Hide();
        }

        private void Zbozí_Load(object sender, EventArgs e)
        {
            // Umístím tlačítka na správná místa.
            this.btnOK.Location = this.btnZavritHist.Location =
              this.btnVlozit.Location;
            this.btnStorno.Location = this.btnUpravit.Location;

            this.NactiData();
        }

        private bool Edit
        {
            get
            {
                return this.bEdit;
            }
            set
            {
                this.bEdit = this.btnOK.Visible = this.btnStorno.Visible = value;

                this.btnHistorie.Visible = this.btnObnovit.Visible =
                  this.chbZrusene.Visible = this.tbNazev.ReadOnly = this.tbPopis.ReadOnly =
                  this.tbCena.ReadOnly = this.nuMnozstvi.ReadOnly = !value;

                this.btnVlozit.Visible = this.btnUpravit.Visible = this.btnNahrad.Visible =
                this.btnZrusit.Visible = !this.chbZrusene.Checked && !value;
            }
        }

        private bool Historie
        {
            get
            {
                return this.bHistorie;
            }
            set
            {
                this.bHistorie = this.btnZavritHist.Visible = value;

                this.btnVlozit.Visible = this.btnUpravit.Visible = this.btnNahrad.Visible =
                  this.btnZrusit.Visible = !this.chbZrusene.Checked && !value;

                this.btnHistorie.Visible = this.chbZrusene.Visible = !value;
            }
        }

        private bool NactiData()
        {
            int iErrKod = 8000;
            FbCommand fbCmd = null;
            FbDataAdapter fbAda = null;
            string sErr, sPole;
            int i1, iRInd, iCInd;

            try
            {
                // Po dobu načítání dat se nebude evidovat změna aktuálního řádku gridu.
                this.bBlokZmenaRadku = true;

                if (!this.util.PripojDb(out fbCmd, out sErr))
                {
                    iErrKod = 8001;
                    this.util.PisLog("Zboží " + iErrKod.ToString() + ": " + sErr,
                      true);
                    return false;
                }

                iRInd = (this.gr?.FirstDisplayedCell?.RowIndex ?? -1);

                iCInd = (this.gr?.FirstDisplayedCell?.ColumnIndex ?? -1);
                fbCmd.Parameters.Clear();
                this.tabZbo.Clear();

                sPole = "KDYZMENIL, ID, NAZEV, POPIS, CENA, MNOZSTVI, DODAVATEL ";

                iErrKod = 8002;
                if (!this.Historie)
                {
                    if (!this.chbZrusene.Checked)
                    {
                        iErrKod = 0003;
                        this.dodavatel.Visible = false;
                        // Všechny záznamy z tabulky TZBOZI.
                        fbCmd.CommandText = "select " + sPole +
                          "from tzbozi order by nazev";

                    }
                    else
                    {
                        iErrKod = 8004;
                        this.dodavatel.Visible = true;
                        /*
                        // Zrušené záznamy (nejsou v tabulce TZBOZI). Vyberu záznamy
                        // s nejnovějším datem změny, tj. datem zrušení.
                        fbCmd.CommandText = "select " + sPole +
                          "from tzbozihis uh where not exists (select 1 " +
                          "from tzbozi u where uh.id = u.id) and " +
                          "uh.kdyzmenil = (select max(kdyzmenil) from tzbozi uh1 " +
                          "where uh.id = uh1.id)";*/

                        // Zrušené záznamy mají v historické tabulce B_ZRUS = 'A'.
                        fbCmd.CommandText = "select " + sPole +
                          "from tzbozihis uh where b_zrus = 'A'";
                    }
                }
                else
                {
                    iErrKod = 8005;
                    this.dodavatel.Visible = true;
                    // Záznamy z tabulek TZBOZI a TZBOZIHIS týkající se daného
                    // id.
                    fbCmd.CommandText = "select " + sPole +
                      "from tzbozi where id = @id " +
                      "union select " + sPole +
                      "from tzbozihis where id = @id " +
                      "order by 1 desc";
                    fbCmd.Parameters.Add("@id", FbDbType.Integer).Value = this.iId;
                }

                fbAda = new FbDataAdapter(fbCmd);
                fbAda.Fill(this.tabZbo);

                // Pokusím se nastavit na stejné místo v gridu jako před načítáním dat.
                if (iRInd >= 0 && iCInd >= 0 && iRInd < this.gr.RowCount &&
                  iCInd < this.gr.ColumnCount)
                    this.gr.FirstDisplayedCell = this.gr[iCInd, iRInd];

                if (this.iId > 0)
                {
                    for (i1 = 0; i1 < this.gr.RowCount &&
                      (int)this.gr["id", i1].Value != this.iId; ++i1) ;

                    if (i1 < this.gr.RowCount)
                        this.gr.CurrentCell =
                          this.gr[(iCInd >= 0 && iCInd < this.gr.ColumnCount ? iCInd : 0),
                          i1];
                }

                return true;
            }
            catch (Exception exc)
            {
                this.util.PisLog("Zboží " + iErrKod.ToString() + ": " +
                  exc.ToString(), true);

                return false;
            }
            finally
            {
                this.bBlokZmenaRadku = false;
                // Musím zajistit aktualizaci iId.
                this.iRowInd = -1;
                this.ZmenaRadku();
                this.Rsz();
                if (fbAda != null)
                    fbAda.Dispose();

                this.util.OdpojDb(fbCmd);
            }
        }

        private void Zbozi_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.F5)
                this.NactiData();
        }

        private void btnObnovit_Click(object sender, EventArgs e)
        {
            this.NactiData();
        }

        private void chbZrusene_CheckedChanged(object sender, EventArgs e)
        {
            this.btnVlozit.Visible = this.btnUpravit.Visible = this.btnNahrad.Visible =
              this.btnZrusit.Visible = !this.chbZrusene.Checked;
            this.NactiData();
        }

        private void btnHistorie_Click(object sender, EventArgs e)
        {
            this.Historie = true;
            this.NactiData();
        }

        private void btnZavritHist_Click(object sender, EventArgs e)
        {
            this.Historie = false;
            this.NactiData();
        }

        private void btnVlozit_Click(object sender, EventArgs e)
        {
            if (!this.util.ClsUziv.bStanice && !this.util.ClsUziv.bAdmin)
            {
                this.util.PisLog("Vkládat zboží může administrátor, nebo správa stanice.", true);
                return;
            }

            this.bEdAdmin = false;
            this.Edit = this.bInsert = true;

            this.tbNazev.Text = this.tbPopis.Text = this.tbCena.Text = this.nuMnozstvi.Text = "";
        }

        private void btnUpravit_Click(object sender, EventArgs e)
        {
            if (!this.util.ClsUziv.bStanice && !this.util.ClsUziv.bAdmin)
            {
                this.util.PisLog("Editovat zboží může administrátor, nebo správa stanice.", true);
                return;
            }

            this.bInsert = false;

            this.Edit = true;
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            this.UlozData();
        }

        private bool UlozData()
        {
            int iErrKod = 8100;
            string sErr;
            FbCommand fbCmd = null;
            FbParameter fbParOut = null;

            try
            {
                iErrKod = 8101;
                if (this.tbNazev.Text == "")
                {
                    this.util.PisLog("Název zboží musí být zadán.", true);
                    this.tbNazev.Focus();
                    return false;
                }

                if (this.tbCena.Text == "")
                {
                    this.util.PisLog("Cena zboží musí být zadána.", true);
                    this.tbNazev.Focus();
                    return false;
                }

                if (this.nuMnozstvi.Text == "")
                {
                    this.util.PisLog("Množství zboží musí být zadáno.", true);
                    this.tbNazev.Focus();
                    return false;
                }

                iErrKod = 8102;
                if (!this.util.PripojDb(out fbCmd, out sErr))
                {
                    iErrKod = 8103;
                    this.util.PisLog("Zboží " + iErrKod.ToString() + ": " + sErr,
                      true);
                    return false;
                }

                if (this.bInsert)
                {
                    iErrKod = 8104;
                    fbCmd.CommandText = "insert into tzbozi (NAZEV, POPIS, " +
                      "CENA, MNOZSTVI, KDOZMENIL) " +
                      "values (@NAZEV, @POPIS, @CENA, @MNOZSTVI, @KDOZMENIL) " +
                      "returning id";

                    // Parametr pro vrácení id vložené věty.
                    fbParOut = new FbParameter();
                    fbParOut.Direction = ParameterDirection.Output;
                    fbParOut.DbType = DbType.Int32;
                    fbParOut.SourceColumn = "id";
                    fbCmd.Parameters.Add(fbParOut);
                }
                else
                {
                    iErrKod = 8105;
                    fbCmd.CommandText = "update tzbozi set NAZEV = @nazev, POPIS = @popis, CENA = @cena, MNOZSTVI = @mnozstvi, " +
                      "KDOZMENIL = @kdozmenil where id = @id";
                    fbCmd.Parameters.Add("@id", FbDbType.Integer).Value = this.iId;
                }

                fbCmd.Parameters.Add("@nazev", FbDbType.VarChar, 100).Value =
                   this.tbNazev.Text.Trim();
                fbCmd.Parameters.Add("@popis", FbDbType.VarChar, 100).Value =
                  this.tbPopis.Text.Trim();
                fbCmd.Parameters.Add("@cena", FbDbType.Integer).Value =
                  this.tbCena.Text.Trim();
                fbCmd.Parameters.Add("@mnozstvi", FbDbType.Integer).Value =
                  this.nuMnozstvi.Text.Trim();
                fbCmd.Parameters.Add("@kdozmenil", FbDbType.VarChar, 200).Value =
                  this.util.KdoZmenil;
                fbCmd.ExecuteNonQuery();

                iErrKod = 8106;
                if (this.bInsert)
                {
                    iErrKod = 8107;
                    this.iId = (int)fbParOut.Value;
                }

                iErrKod = 8108;
                fbCmd.Transaction.Commit();

                this.bZmena = this.Edit = false;
                this.NactiData();
                return true;
            }
            catch (Exception exc)
            {
                this.util.PisLog("Zboží " + iErrKod.ToString() + ". " +
                  exc.ToString(), true);
                return false;
            }
            finally
            {
                this.util.OdpojDb(fbCmd);
            }
        }

        private void btnZrusit_Click(object sender, EventArgs e)
        {
            int iErrKod = 8200;
            FbCommand fbCmd = null;
            string sErr;

            try
            {
                if (!this.util.ClsUziv.bStanice && !this.util.ClsUziv.bAdmin)
                {
                    this.util.PisLog("Rušit zboží může administrátor, nebo správa stanice.", true);
                    return;
                }

                iErrKod = 8201;
                if (this.util.PisLog("Opravdu chcete zrušit zboží \"" +
                  this.tbNazev.Text + "\"?", true, MessageBoxButtons.YesNo,
                  MessageBoxIcon.Question) != DialogResult.Yes)
                    return;

                iErrKod = 8202;
                if (!this.util.PripojDb(out fbCmd, out sErr))
                {
                    iErrKod = 8203;
                    this.util.PisLog("Zboží " + iErrKod.ToString() + ": " + sErr,
                      true);
                    return;
                }

                fbCmd.CommandText = "update tzbozi set b_zrus = 'A', " +
                  "kdozmenil = @kdozmenil where id = @id";
                fbCmd.Parameters.Add("@kdozmenil", FbDbType.VarChar, 2000).Value =
                  this.util.KdoZmenil;
                fbCmd.Parameters.Add("@id", FbDbType.Integer).Value = this.iId;
                fbCmd.ExecuteNonQuery();
                fbCmd.Transaction.Commit();

                // Po zrušení zboží bych se chtěl nastavit na následující nebo
                // předcházející řádek v gridu.
                if (this.gr.RowCount > this.gr.CurrentRow.Index + 1)
                    this.iId = (int)this.gr["id", this.gr.CurrentRow.Index + 1].Value;
                else
                if (this.gr.CurrentRow.Index > 0)
                    this.iId = (int)this.gr["id", this.gr.CurrentRow.Index - 1].Value;

                this.NactiData();
            }
            catch (Exception exc)
            {
                this.util.PisLog("Uzivatele " + iErrKod.ToString() + ": " +
                  exc.ToString(), true);
            }
            finally
            {
                this.util.OdpojDb(fbCmd);
            }
        }

        private void btnStorno_Click(object sender, EventArgs e)
        {
            this.bZmena = this.Edit = false;
            // Změním index řádku, aby se do detailu načetla aktuální věta z gridu.
            this.iRowInd = -2;
            this.ZmenaRadku();
        }

        private void gr_CurrentCellChanged(object sender, EventArgs e)
        {
            this.ZmenaRadku();
        }

        /// <summary>
        /// Obsluha změny aktuálního řádku v gridu.
        /// </summary>
        private void ZmenaRadku()
        {
            // Změnu budu evidovat jen v případě, že opravdu dojde ke změně řádku.
            if (this.bBlokZmenaRadku || this.Edit || this.gr.CurrentRow == null ||
              this.gr.CurrentRow.Index == this.iRowInd)
                return;

            this.iRowInd = this.gr.CurrentRow.Index;
            this.iId = (int)this.gr["id", iRowInd].Value;

            if (!this.Edit)
            {
                this.tbNazev.Text = (string)this.gr.CurrentRow.Cells["nazev"].Value;
                this.tbPopis.Text = (string)this.gr.CurrentRow.Cells["popis"].Value;
                this.tbCena.Text = Convert.ToString(this.gr.CurrentRow.Cells["cena"].Value);
                this.nuMnozstvi.Text = Convert.ToString(this.gr.CurrentRow.Cells["mnozstvi"].Value);
            }
        }

        private void gr_Resize(object sender, EventArgs e)
        {
            this.Rsz();
        }

        private void Rsz()
        {
            // Sloupec "popis" bude vyplňovat zbytek gridu.
            if(this.chbZrusene.Checked || this.Historie)
            {
                this.popis.Width = this.gr.ClientSize.Width - this.nazev.Width - this.cena.Width - this.mnozstvi.Width - this.dodavatel.Width - 25;
            }
            else
                this.popis.Width = this.gr.ClientSize.Width - this.nazev.Width - this.cena.Width - this.mnozstvi.Width - 25;
        }

        private void btnNahrat_Click(object sender, EventArgs e)
        {
            if (this.bw.IsBusy)
            {
                this.util.PisLog("Právě probíhá nahrávání číselníků.", true);
                return;
            }

            if (!this.util.ClsUziv.bAdmin)
            {
                this.util.PisLog("Číselníky může nahrávat jen administrátor.", true);
                return;
            }

            if (this.openDlg.ShowDialog() != DialogResult.OK)
                return;

            // Číselníky se budou ze souboru načítat na pozadí, protože to dlouho 
            // trvá.
            this.sSoub = this.openDlg.FileName;
            this.sbErr.Clear();
            this.bw.RunWorkerAsync();
        }

        /// <summary>
        /// Načítání zboží na pozadí.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bw_DoWork(object sender, DoWorkEventArgs e)
        {
            int iErrKod = 8300;
            FbCommand fbCmd = null;
            string sErr;
            string[] arrRadky, arrUdaje;
            bool bPrvni = true;
            int iPocet = 0;

            try
            {
                iErrKod = 8301;
                arrRadky = File.ReadAllLines(this.sSoub, this.enc);

                iErrKod = 8002;
                if (!this.util.PripojDb(out fbCmd, out sErr))
                {
                    iErrKod = 8003;
                    this.sbErr.AppendLine("Zboží " + iErrKod.ToString() +
                      ": " + sErr);
                    return;
                }

                iErrKod = 8004;

                fbCmd.CommandText = "delete from  tzbozi";
                fbCmd.ExecuteNonQuery();

                // Protože ruším celou tabulku, můžu resetovat i sekvenci, aby 
                // při opakovaném nahrávání číselníků časem nepřetekla.
                iErrKod = 8005;
                fbCmd.CommandText = "ALTER SEQUENCE g_zbozi_id RESTART WITH 0";
                fbCmd.ExecuteNonQuery();

                iErrKod = 8006;
                fbCmd.CommandText = "insert into tzbozi (nazev, popis, cena, mnozstvi) " +
                  "values (@nazev, @popis, @cena, @mnozstvi)";
                fbCmd.Parameters.Add("@nazev", FbDbType.VarChar, 100);
                fbCmd.Parameters.Add("@popis", FbDbType.VarChar, 100);
                fbCmd.Parameters.Add("@cena", FbDbType.Integer);
                fbCmd.Parameters.Add("@mnozstvi", FbDbType.Integer);

                foreach (string s in arrRadky)
                {
                    if (this.bw.CancellationPending)
                    {
                        this.sbErr.AppendLine("Nahrávání zboží bylo předčasně ukončeno.");
                        return;
                    }

                    if ((++iPocet) % 100 == 0)
                        this.bw.ReportProgress(iPocet);

                    arrUdaje = this.util.RozlozCSV(s);

                    // V 1. sloupci název, ve 2. popis, ve 3. cena a ve 4. množství.
                    fbCmd.Parameters["@nazev"].Value = arrUdaje[0];
                    fbCmd.Parameters["@popis"].Value = arrUdaje[1];
                    fbCmd.Parameters["@cena"].Value = arrUdaje[2];
                    fbCmd.Parameters["@mnozstvi"].Value = arrUdaje[3];

                    fbCmd.ExecuteNonQuery();
                }

                fbCmd.Transaction.Commit();
            }
            catch (Exception exc)
            {
                this.sbErr.AppendLine("Zboží " + iErrKod.ToString() + ": " +
                  exc.ToString());
            }
            finally
            {
                this.util.OdpojDb(fbCmd);
            }
        }

        private void bw_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            this.labPocet.Text = "Nahráno " + e.ProgressPercentage.ToString("#,0") +
              " vět.";
        }

        private void bw_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.labPocet.Text = "";

            if (this.sbErr.Length > 0)
            {
                this.util.PisLog("Při nahrávání zboží \"" +
                  "\" došlo k chybám: " + this.sbErr.ToString(), true);
            }
            else
            {
                this.util.PisLog("Zboží \"" +
                  "\" se úspěšně nahrálo do databáze.", true, MessageBoxButtons.OK,
                  MessageBoxIcon.Information);
            }
        }
    }
}
